require 'csv'
require 'sketchup.rb'
require 'extensions.rb'

module ImportProjectPricePlan

  # Function to import points and form shapes
  def self.import_points(file_path)
    model = Sketchup.active_model
    entities = model.entities
    materials = model.materials

    points_hash = Hash.new { |hash, key| hash[key] = [] }

    begin
      # Read points from CSV file and store them in a hash by shape
      CSV.foreach(file_path, headers: false) do |row|
        shape, x, y = row
        x = x.to_f
        y = y.to_f
        z = 0.0 # Default Z-coordinate

        point = Geom::Point3d.new(x, y, z)
        points_hash[shape] << point
      end

      # Calculate the centroid of all points
      all_points = points_hash.values.flatten(1)
      total_points = all_points.size
      centroid = all_points.inject([0, 0, 0]) { |sum, point| [sum[0] + point.x, sum[1] + point.y, sum[2] + point.z] }.map { |coord| coord / total_points.to_f }

      # Translate all points to center them around the origin
      transformation = Geom::Transformation.new([-centroid[0], -centroid[1], -centroid[2]])
      points_hash.each do |shape, points|
        points.map! { |point| point.transform(transformation) }
      end

      # Create faces (shapes) from the points
      points_hash.each do |shape, points|
        if points.length > 2
          face = entities.add_face(points)
          if face
            face.reverse! if face.normal.z < 0

            # Generate a unique material name for each shape
            material_name = "#{shape}_#{points_hash.keys.count(shape)}"

            # Extract base shape name using regular expressions
            base_shape = shape[/[a-zA-Z]+/]

            # Apply textures based on base shape type
            texture_path = case base_shape
                           when 'patio'
                             File.join(__dir__, 'textures', 'patio.png')
                           when 'decking'
                             File.join(__dir__, 'textures', 'decking.png')
                           when 'edgings'
                             File.join(__dir__, 'textures', 'edgings.png')
                           when 'asphalt'
                             File.join(__dir__, 'textures', 'asphalt.png')
                           when 'bark'
                             File.join(__dir__, 'textures', 'bark.png')
                           when 'blockpaving'
                             File.join(__dir__, 'textures', 'blockpaving.png')
                           when 'concretepad'
                             File.join(__dir__, 'textures', 'concrete.png')
                           when 'gravel'
                             File.join(__dir__, 'textures', 'gravel.png')
                           when 'setts'
                             File.join(__dir__, 'textures', 'setts.png')
                           when 'artificialturf', 'turf'
                             File.join(__dir__, 'textures', 'grass.png')
                           when 'sleepers'
                             File.join(__dir__, 'textures', 'sleepers.png')
                           when 'curb'
                             File.join(__dir__, 'textures', 'curb.png')
                           when 'walling'
                             File.join(__dir__, 'textures', 'walling.png')
                           when 'channeldrain'
                             File.join(__dir__, 'textures', 'channeldrain.png')
                           when 'fencing'
                             File.join(__dir__, 'textures', 'fencing.png')
                           # Add cases for other shapes as needed
                           else
                             nil
                           end

            if texture_path && File.exist?(texture_path)
              material = materials.add(material_name)
              material.texture = texture_path

              # Set the texture size based on shape type
              texture_size = case base_shape
                             when 'patio'
                               145 # Set texture size to 117 for patios
                             when 'fencing'
                               120 # Set texture size to 120 for fencing
                             when 'decking'
                               8 # Set texture size to 5 for decking
                             when 'turf'
                               145 # Set texture size to 131 for turf
                             when 'asphalt'
                               60 # Set texture size to 60 for asphalt
			     when 'walling'
                               [20, 6] # Set texture size to 60 width and 30 height for walling
                             when 'gravel'
                              15 # Set texture size to 10 for gravel
                             else
                               38 # Default texture size for other shapes
                             end

              # Log diagnostic information
              puts "Applying texture '#{texture_path}' to shape '#{shape}' with bounding box (W: #{face.bounds.width}, H: #{face.bounds.height}) and texture size #{texture_size}"

              # Apply texture scaling
              if base_shape == 'walling'
                material.texture.size = texture_size
              else
                material.texture.size = [texture_size, texture_size]
              end
              face.material = material
              face.back_material = material

              # Verify and log the texture size applied
              applied_texture_width = material.texture.width
              applied_texture_height = material.texture.height
              puts "Verified texture size for shape '#{shape}': Width = #{applied_texture_width}, Height = #{applied_texture_height}"
            else
              puts "Texture not found for shape '#{shape}' or texture path does not exist"
            end
          else
            puts "Failed to create face for shape '#{shape}'"
          end
        else
          puts "Not enough points to form a face for shape: #{shape}"
        end
      end
    rescue => e
      puts "Error processing file: #{e.message}"
      puts e.backtrace.join("\n")
    end
  end

  # Prompt user to select CSV file without scale factor prompt
  unless file_loaded?(__FILE__)
    UI.menu("Plugins").add_item("Import Project Price Plan") do
      file_path = UI.openpanel("Select CSV File", "", "CSV Files|*.csv||")
      if file_path
        puts "File selected: #{file_path}"
        import_points(file_path)
      else
        puts "No file selected."
      end
    end
    file_loaded(__FILE__)
  end

end
